@@ -17,10 +17,6 @@ If n is the modulus, all inputs should be in the range 0..n-1.
 The number n itself should be in the range 1..2^{NTL_SP_NBITS}-1.
 */
 
-// I've declared these "static" so that the installation wizard
-// has more flexibility, without worrying about the (rather esoteric)
-// possibility of the linker complaining when the definitions
-// are inconsistent across several files.
 
 // DIRT: undocumented feature: in all of these MulMod routines,
 // the first argument, a, need only be in the range
@@ -42,6 +38,16 @@ NTL_OPEN_NNS
 
 #define NTL_HAVE_MULMOD_T
 
+#if (defined(NTL_SPMM_ULL) && defined(NTL_HAVE_LL_TYPE))
+// we only honor SPMM_ULL if we have LL
+#define NTL_SPMM_ULL_VIABLE
+
+#elif (defined(NTL_SPMM_ULL) && defined(NTL_WIZARD_HACK))
+// raise an error when running the wizard and we cannot honor SPMM_ULL
+#error "cannot honor NTL_SPMM_ULL"
+
+#endif
+
 
 
 #if 0
@@ -57,23 +63,23 @@ typedef double mulmod_t;
 typedef double muldivrem_t;
 
 
-static inline double PrepMulMod(long n)
+inline double PrepMulMod(long n)
 {
    return double(1L)/double(n);
 }
 
-static inline double PrepMulDivRem(long b, long n, double ninv)
+inline double PrepMulDivRem(long b, long n, double ninv)
 {
    return double(b)*ninv;
 }
 
-static inline double PrepMulDivRem(long b, long n)
+inline double PrepMulDivRem(long b, long n)
 {
    return double(b)/double(n);
 }
 
 
-static inline double PrepMulModPrecon(long b, long n)
+inline double PrepMulModPrecon(long b, long n)
 {
    return PrepMulModPrecon(b, n, PrepMulMod(n));
 }
@@ -114,7 +120,7 @@ long sp_CorrectDeficit(unsigned long a, long n):
 // and that long(a) >= -n 
 
 long sp_CorrectExcess(long a, long n) 
-long sp_CorrectDeficit(unsigned long a, long n): 
+long sp_CorrectExcess(unsigned long a, long n): 
 // if (a < n) then a else a-n
 
 // For the signed version, it is assumed that a >= 0.
@@ -132,35 +138,35 @@ NTL_ARITH_RIGHT_SHIFT, and NTL_AVOID_BRANCHING.
 #if (NTL_ARITH_RIGHT_SHIFT && !defined(NTL_CLEAN_INT))
 // DIRT: IMPL-DEF: arithmetic right shift and cast unsigned to signed
 
-static inline 
+inline 
 long sp_SignMask(long a)
 {
    return a >> (NTL_BITS_PER_LONG-1);
 }
 
-static inline 
+inline 
 long sp_SignMask(unsigned long a)
 {
    return cast_signed(a) >> (NTL_BITS_PER_LONG-1);
 }
 #else
-static inline 
+inline 
 long sp_SignMask(long a)
 {
    return -long(cast_unsigned(a) >> (NTL_BITS_PER_LONG-1));
 }
 
-static inline 
+inline 
 long sp_SignMask(unsigned long a)
 {
    return -long(a >> (NTL_BITS_PER_LONG-1));
 }
 #endif
 
-static inline
+inline
 bool sp_Negative(unsigned long a)
 {
-   return clean_cast_signed(a) < 0;
+   return cast_signed(a) < 0;
 }
 
 
@@ -174,13 +180,13 @@ bool sp_Negative(unsigned long a)
 // in written in a particular way to get optimal machine code:
 // subtract, cmove (tested on clang, gcc, icc).
 
-static inline 
+inline 
 long sp_CorrectDeficit(long a, long n)
 {
    return a >= 0 ? a : a+n;
 }
 
-template<class T> static inline 
+template<class T> 
 long sp_CorrectDeficitQuo(T& q, long a, long n, long amt=1)
 {
    return a >= 0 ? a : (q -= amt, a+n);
@@ -188,13 +194,13 @@ long sp_CorrectDeficitQuo(T& q, long a, long n, long amt=1)
 
 
 
-static inline 
+inline 
 long sp_CorrectDeficit(unsigned long a, long n)
 {
    return !sp_Negative(a) ? a : a+n;
 }
 
-template<class T> static inline 
+template<class T> 
 long sp_CorrectDeficitQuo(T& q, unsigned long a, long n, long amt=1)
 {
    return !sp_Negative(a) ? a : (q -= amt, a+n);
@@ -202,13 +208,13 @@ long sp_CorrectDeficitQuo(T& q, unsigned long a, long n, long amt=1)
 
 
 
-static inline 
+inline 
 long sp_CorrectExcess(long a, long n)
 {
    return a-n >= 0 ? a-n : a;
 }
 
-template<class T> static inline 
+template<class T> 
 long sp_CorrectExcessQuo(T& q, long a, long n, long amt=1)
 {
    return a-n >= 0 ? (q += amt, a-n) : a;
@@ -216,13 +222,13 @@ long sp_CorrectExcessQuo(T& q, long a, long n, long amt=1)
 
 
 
-static inline 
+inline 
 long sp_CorrectExcess(unsigned long a, long n)
 {
    return !sp_Negative(a-n) ? a-n : a;
 }
 
-template<class T> static inline 
+template<class T> 
 long sp_CorrectExcessQuo(T& q, unsigned long a, long n, long amt=1)
 {
    return !sp_Negative(a-n) ? (q += amt, a-n) : a;
@@ -234,13 +240,13 @@ long sp_CorrectExcessQuo(T& q, unsigned long a, long n, long amt=1)
 // This C++ code uses traditional masking and adding
 // to avoid branching. 
 
-static inline 
+inline 
 long sp_CorrectDeficit(long a, long n)
 {
    return a + (sp_SignMask(a) & n);
 }
 
-template<class T> static inline 
+template<class T> 
 long sp_CorrectDeficitQuo(T& q, long a, long n, long amt=1)
 {
    q += sp_SignMask(a)*amt;
@@ -251,13 +257,13 @@ long sp_CorrectDeficitQuo(T& q, long a, long n, long amt=1)
 
 
 
-static inline 
+inline 
 long sp_CorrectDeficit(unsigned long a, long n)
 {
    return a + (sp_SignMask(a) & n);
 }
 
-template<class T> static inline 
+template<class T> 
 long sp_CorrectDeficitQuo(T& q, unsigned long a, long n, long amt=1)
 {
    q += sp_SignMask(a)*amt;
@@ -267,13 +273,13 @@ long sp_CorrectDeficitQuo(T& q, unsigned long a, long n, long amt=1)
 
 
 
-static inline 
+inline 
 long sp_CorrectExcess(long a, long n)
 {
    return (a-n) + (sp_SignMask(a-n) & n);
 }
 
-template<class T> static inline 
+template<class T> 
 long sp_CorrectExcessQuo(T& q, long a, long n, long amt=1)
 {
    q += (1L + sp_SignMask(a-n))*amt;
@@ -283,13 +289,13 @@ long sp_CorrectExcessQuo(T& q, long a, long n, long amt=1)
 
 
 
-static inline 
+inline 
 long sp_CorrectExcess(unsigned long a, long n)
 {
    return (a-n) + (sp_SignMask(a-n) & n);
 }
 
-template<class T> static inline 
+template<class T> 
 long sp_CorrectExcessQuo(T& q, unsigned long a, long n, long amt=1)
 {
    q += (1L + sp_SignMask(a-n))*amt;
@@ -307,7 +313,7 @@ long sp_CorrectExcessQuo(T& q, unsigned long a, long n, long amt=1)
 
 #ifdef NTL_HAVE_BUILTIN_CLZL
 
-static inline long 
+inline long 
 sp_CountLeadingZeros(unsigned long x)
 {
    return __builtin_clzl(x);
@@ -315,11 +321,11 @@ sp_CountLeadingZeros(unsigned long x)
 
 #else
 
-static inline long 
+inline long 
 sp_CountLeadingZeros(unsigned long x)
 {
    long res = NTL_BITS_PER_LONG-NTL_SP_NBITS;
-   x = x << NTL_BITS_PER_LONG-NTL_SP_NBITS;
+   x = x << (NTL_BITS_PER_LONG-NTL_SP_NBITS);
    while (x < (1UL << (NTL_BITS_PER_LONG-1))) {
       x <<= 1;
       res++;
@@ -334,45 +340,26 @@ sp_CountLeadingZeros(unsigned long x)
 
 
 
-static inline 
+inline 
 long AddMod(long a, long b, long n)
 {
    long r = a+b;
    return sp_CorrectExcess(r, n);
 }
 
-static inline 
+inline 
 long SubMod(long a, long b, long n)
 {
    long r = a-b;
    return sp_CorrectDeficit(r, n);
 }
 
-static inline 
+inline 
 long NegateMod(long a, long n)
 {
    return SubMod(0, a, n);
 }
 
-// definition of MulhiUL, using either assembly or ULL type
-#if (defined(NTL_SPMM_ASM))
-#define NTL_HAVE_MULHI
-
-// assmbly code versions
-#include <NTL/SPMM_ASM.h>
-
-
-#elif (defined(NTL_SPMM_ULL) || defined(NTL_HAVE_LL_TYPE))
-#define NTL_HAVE_MULHI
-
-static inline unsigned long 
-MulHiUL(unsigned long a, unsigned long b)
-{
-   return (((NTL_ULL_TYPE)(a)) * ((NTL_ULL_TYPE)(b))) >> NTL_BITS_PER_LONG;
-} 
-#endif
-
-
 
 
 
@@ -481,18 +468,18 @@ inline wide_double& operator/=(wide_double& x, wide_double y)
 typedef wide_double mulmod_t;
 typedef wide_double muldivrem_t;
 
-static inline wide_double PrepMulMod(long n)
+inline wide_double PrepMulMod(long n)
 {
    return wide_double(1L)/wide_double(n);
 }
 
-static inline wide_double PrepMulDivRem(long b, long n, wide_double ninv)
+inline wide_double PrepMulDivRem(long b, long n, wide_double ninv)
 {
    return wide_double(b)*ninv;
 }
 
 
-static inline 
+inline 
 long MulMod(long a, long b, long n, wide_double ninv)
 {
    long q  = (long) ((((wide_double) a) * ((wide_double) b)) * ninv); 
@@ -502,17 +489,17 @@ long MulMod(long a, long b, long n, wide_double ninv)
    return sp_CorrectExcess(r, n);
 }
 
-static inline 
+inline 
 long NormalizedMulMod(long a, long b, long n, wide_double ninv)
 {
    return MulMod(a, b, n, ninv);
 }
 
-static inline bool NormalizedModulus(wide_double ninv) { return true; }
+inline bool NormalizedModulus(wide_double ninv) { return true; }
 
 
 
-static inline 
+inline 
 long MulModWithQuo(long& qres, long a, long b, long n, wide_double ninv)
 {
    long q  = (long) ((((wide_double) a) * ((wide_double) b)) * ninv); 
@@ -526,7 +513,7 @@ long MulModWithQuo(long& qres, long a, long b, long n, wide_double ninv)
 }
 
 
-static inline 
+inline 
 long MulMod2_legacy(long a, long b, long n, wide_double bninv)
 {
    long q  = (long) (((wide_double) a) * bninv);
@@ -537,7 +524,7 @@ long MulMod2_legacy(long a, long b, long n, wide_double bninv)
    return r;
 }
 
-static inline 
+inline 
 long MulDivRem(long& qres, long a, long b, long n, wide_double bninv)
 {
    long q  = (long) (((wide_double) a) * bninv);
@@ -560,7 +547,7 @@ struct sp_inverse {
    unsigned long inv;
    long shamt;
 
-   sp_inverse() { }
+   sp_inverse() NTL_DEFAULT
    sp_inverse(unsigned long _inv, long _shamt) : inv(_inv), shamt(_shamt) { }
 };
 
@@ -590,7 +577,7 @@ typedef sp_inverse mulmod_t;
 #if (NTL_SP_NBITS <= 2*NTL_DOUBLE_PRECISION-10)
 
 
-static inline unsigned long
+inline unsigned long
 sp_NormalizedPrepMulMod(long n)
 {
    double ninv = 1/double(n); 
@@ -617,8 +604,8 @@ sp_NormalizedPrepMulMod(long n)
    unsigned long rrhi = ((unsigned long) (rr >> NTL_BITS_PER_LONG)) 
                         + (rrlo >> (NTL_BITS_PER_LONG-1));
 
-   long rlo = clean_cast_signed(rrlo);  // these should be No-Ops
-   long rhi = clean_cast_signed(rrhi);
+   long rlo = cast_signed(rrlo);  // these should be No-Ops
+   long rhi = cast_signed(rrhi);
 
    const double bpl_as_double (double(1L << NTL_SP_NBITS) * double(1L << (NTL_BITS_PER_LONG-NTL_SP_NBITS)));
    double fr = double(rlo) + double(rhi)*bpl_as_double;
@@ -647,7 +634,7 @@ sp_NormalizedPrepMulMod(long n)
 
 #else
 
-static inline unsigned long 
+inline unsigned long 
 sp_NormalizedPrepMulMod(long n)
 {
    return 
@@ -657,7 +644,7 @@ sp_NormalizedPrepMulMod(long n)
 #endif
 
 
-static inline sp_inverse
+inline sp_inverse
 PrepMulMod(long n)
 {
    long shamt = sp_CountLeadingZeros(n) - (NTL_BITS_PER_LONG-NTL_SP_NBITS);
@@ -671,21 +658,24 @@ PrepMulMod(long n)
 
 
 
-static inline long 
+inline long 
 sp_NormalizedMulMod(long a, long b, long n, unsigned long ninv) 
 {
-   NTL_ULL_TYPE U = ((NTL_ULL_TYPE) cast_unsigned(a)) * ((NTL_ULL_TYPE) cast_unsigned(b));
-   unsigned long H = U >> (NTL_SP_NBITS-2);
-   unsigned long Q = MulHiUL(H, ninv);
+   ll_type U;
+   ll_imul(U, a, b);
+   unsigned long H = ll_rshift_get_lo<NTL_SP_NBITS-2>(U);
+   unsigned long Q = ll_mul_hi(H, ninv);
    Q = Q >> NTL_POST_SHIFT;
-   unsigned long L = U;
+   unsigned long L = ll_get_lo(U);
    long r = L - Q*cast_unsigned(n);  // r in [0..2*n)
 
    r = sp_CorrectExcess(r, n);
    return r;
 }
 
-static inline long 
+
+
+inline long 
 MulMod(long a, long b, long n, sp_inverse ninv)
 {
    return sp_NormalizedMulMod(a, b << ninv.shamt, n << ninv.shamt, ninv.inv) >> ninv.shamt;
@@ -694,26 +684,27 @@ MulMod(long a, long b, long n, sp_inverse ninv)
 // if you know what you're doing....
 // FIXME: eventually, put this is the documented interface...
 // but for now, it's "experimental"
-static inline long 
+inline long 
 NormalizedMulMod(long a, long b, long n, sp_inverse ninv)
 {
    return sp_NormalizedMulMod(a, b, n, ninv.inv);
 }
 
-static inline bool
+inline bool
 NormalizedModulus(sp_inverse ninv) { return ninv.shamt == 0; }
 
 
 
 
-static inline long 
+inline long 
 sp_NormalizedMulModWithQuo(long& qres, long a, long b, long n, unsigned long ninv)
 {
-   NTL_ULL_TYPE U = ((NTL_ULL_TYPE) cast_unsigned(a)) * ((NTL_ULL_TYPE) cast_unsigned(b));
-   unsigned long H = U >> (NTL_SP_NBITS-2);
-   unsigned long Q = MulHiUL(H, ninv);
+   ll_type U;
+   ll_imul(U, a, b);
+   unsigned long H = ll_rshift_get_lo<NTL_SP_NBITS-2>(U);
+   unsigned long Q = ll_mul_hi(H, ninv);
    Q = Q >> NTL_POST_SHIFT;
-   unsigned long L = U;
+   unsigned long L = ll_get_lo(U);
    long r = L - Q*cast_unsigned(n);  // r in [0..2*n)
 
    r = sp_CorrectExcessQuo(Q, r, n);
@@ -721,7 +712,7 @@ sp_NormalizedMulModWithQuo(long& qres, long a, long b, long n, unsigned long nin
    return r;
 }
 
-static inline long 
+inline long 
 MulModWithQuo(long& qres, long a, long b, long n, sp_inverse ninv)
 {
    return sp_NormalizedMulModWithQuo(qres, a, b << ninv.shamt, n << ninv.shamt, ninv.inv) >> ninv.shamt;
@@ -734,7 +725,7 @@ MulModWithQuo(long& qres, long a, long b, long n, sp_inverse ninv)
 
 
 
-#if (defined(NTL_SPMM_ULL) || defined(NTL_SPMM_ASM) || defined(NTL_LONGLONG_SP_MULMOD))
+#if (defined(NTL_SPMM_ULL_VIABLE) || defined(NTL_LONGLONG_SP_MULMOD))
 
 
 typedef unsigned long mulmod_precon_t;
@@ -742,7 +733,7 @@ typedef unsigned long mulmod_precon_t;
 
 #if (!defined(NTL_LONGLONG_SP_MULMOD))
 
-static inline unsigned long PrepMulModPrecon(long b, long n, wide_double ninv)
+inline unsigned long PrepMulModPrecon(long b, long n, wide_double ninv)
 {
    long q  = (long) ( (((wide_double) b) * wide_double(NTL_SP_BOUND)) * ninv ); 
    unsigned long rr = (cast_unsigned(b) << NTL_SP_NBITS) - cast_unsigned(q)*cast_unsigned(n);
@@ -754,14 +745,14 @@ static inline unsigned long PrepMulModPrecon(long b, long n, wide_double ninv)
 
 #else
 
-static inline unsigned long 
+
+inline unsigned long 
 sp_NormalizedPrepMulModPrecon(long b, long n, unsigned long ninv)
 {
-   NTL_ULL_TYPE U = ((NTL_ULL_TYPE) cast_unsigned(b)) << NTL_SP_NBITS;
-   unsigned long H = U >> (NTL_SP_NBITS-2);
-   unsigned long Q = MulHiUL(H, ninv);
+   unsigned long H = cast_unsigned(b) << 2;
+   unsigned long Q = ll_mul_hi(H, ninv);
    Q = Q >> NTL_POST_SHIFT;
-   unsigned long L = U;
+   unsigned long L = cast_unsigned(b) << NTL_SP_NBITS;
    long r = L - Q*cast_unsigned(n);  // r in [0..2*n)
 
 
@@ -769,7 +760,8 @@ sp_NormalizedPrepMulModPrecon(long b, long n, unsigned long ninv)
    return Q;  // NOTE: not shifted
 }
 
-static inline unsigned long 
+
+inline unsigned long 
 PrepMulModPrecon(long b, long n, sp_inverse ninv)
 {
    return sp_NormalizedPrepMulModPrecon(b << ninv.shamt, n << ninv.shamt, ninv.inv) << (NTL_BITS_PER_LONG-NTL_SP_NBITS);
@@ -785,18 +777,18 @@ PrepMulModPrecon(long b, long n, sp_inverse ninv)
    
 
 
-static inline long MulModPrecon(long a, long b, long n, unsigned long bninv)
+inline long MulModPrecon(long a, long b, long n, unsigned long bninv)
 {
-   unsigned long qq = MulHiUL(a, bninv);
+   unsigned long qq = ll_mul_hi(a, bninv);
    unsigned long rr = cast_unsigned(a)*cast_unsigned(b) - qq*cast_unsigned(n);
    return sp_CorrectExcess(long(rr), n);
 }
 
 
 
-static inline long MulModPreconWithQuo(long& qres, long a, long b, long n, unsigned long bninv)
+inline long MulModPreconWithQuo(long& qres, long a, long b, long n, unsigned long bninv)
 {
-   unsigned long qq = MulHiUL(a, bninv);
+   unsigned long qq = ll_mul_hi(a, bninv);
    unsigned long rr = cast_unsigned(a)*cast_unsigned(b) - qq*cast_unsigned(n);
    long r = sp_CorrectExcessQuo(qq, long(rr), n);
    qres = long(qq);
@@ -812,17 +804,17 @@ static inline long MulModPreconWithQuo(long& qres, long a, long b, long n, unsig
 typedef wide_double mulmod_precon_t;
 
 
-static inline wide_double PrepMulModPrecon(long b, long n, wide_double ninv)
+inline wide_double PrepMulModPrecon(long b, long n, wide_double ninv)
 {
    return ((wide_double) b) * ninv;
 }
 
-static inline long MulModPrecon(long a, long b, long n, wide_double bninv)
+inline long MulModPrecon(long a, long b, long n, wide_double bninv)
 {
    return MulMod2_legacy(a, b, n, bninv);
 }
 
-static inline long MulModPreconWithQuo(long& qq, long a, long b, long n, wide_double bninv)
+inline long MulModPreconWithQuo(long& qq, long a, long b, long n, wide_double bninv)
 {
    return MulDivRem(qq, a, b, n, bninv);
 }
@@ -846,18 +838,18 @@ struct sp_muldivrem_struct {
    unsigned long bninv;
 
    explicit sp_muldivrem_struct(unsigned long _bninv) : bninv(_bninv) { }
-   sp_muldivrem_struct() { }
+   sp_muldivrem_struct() NTL_DEFAULT
 };
 
 typedef sp_muldivrem_struct muldivrem_t;
 
-static inline sp_muldivrem_struct PrepMulDivRem(long b, long n, sp_inverse ninv)
+inline sp_muldivrem_struct PrepMulDivRem(long b, long n, sp_inverse ninv)
 {
    return sp_muldivrem_struct(PrepMulModPrecon(b, n, ninv));
 }
 
 
-static inline
+inline
 long MulDivRem(long& qres, long a, long b, long n,  sp_muldivrem_struct bninv)
 {
    return MulModPreconWithQuo(qres, a, b, n, bninv.bninv);
@@ -870,19 +862,19 @@ long MulDivRem(long& qres, long a, long b, long n,  sp_muldivrem_struct bninv)
 
 
 
-static inline mulmod_precon_t PrepMulModPrecon(long b, long n)
+inline mulmod_precon_t PrepMulModPrecon(long b, long n)
 {
    return PrepMulModPrecon(b, n, PrepMulMod(n));
 }
 
 
-static inline 
+inline 
 long MulMod(long a, long b, long n)
 {
    return MulMod(a, b, n, PrepMulMod(n));
 }
 
-static inline muldivrem_t PrepMulDivRem(long b, long n)
+inline muldivrem_t PrepMulDivRem(long b, long n)
 {
    return PrepMulDivRem(b, n, PrepMulMod(n));
 }
@@ -894,7 +886,7 @@ static inline muldivrem_t PrepMulDivRem(long b, long n)
 
 #ifdef NTL_LEGACY_SP_MULMOD
 
-static inline long MulMod2(long a, long b, long n, wide_double bninv)
+inline long MulMod2(long a, long b, long n, wide_double bninv)
 {
    return MulMod2_legacy(a, b, n, bninv);
 }
@@ -905,7 +897,7 @@ static inline long MulMod2(long a, long b, long n, wide_double bninv)
 
 
 
-static inline
+inline
 void VectorMulModPrecon(long k, long *x, const long *a, long b, long n, 
                         mulmod_precon_t bninv)
 {
@@ -913,7 +905,7 @@ void VectorMulModPrecon(long k, long *x, const long *a, long b, long n,
       x[i] = MulModPrecon(a[i], b, n, bninv);
 }
 
-static inline
+inline
 void VectorMulMod(long k, long *x, const long *a, long b, long n, 
                   mulmod_t ninv)
 {
@@ -923,14 +915,14 @@ void VectorMulMod(long k, long *x, const long *a, long b, long n,
 }
 
 
-static inline 
+inline 
 void VectorMulMod(long k, long *x, const long *a, long b, long n)
 {
    mulmod_t ninv = PrepMulMod(n);
    VectorMulMod(k, x, a, b, n, ninv);
 }
 
-#ifdef NTL_HAVE_MULHI
+#ifdef NTL_HAVE_LL_TYPE
 
 
 struct sp_reduce_struct {
@@ -940,10 +932,10 @@ struct sp_reduce_struct {
    sp_reduce_struct(unsigned long _ninv, long _sgn) : 
       ninv(_ninv), sgn(_sgn)  { }
 
-   sp_reduce_struct() { }
+   sp_reduce_struct() NTL_DEFAULT
 };
 
-static inline
+inline
 sp_reduce_struct sp_PrepRem(long n)
 {
    unsigned long q = (1UL << (NTL_BITS_PER_LONG-1))/cast_unsigned(n);
@@ -958,17 +950,17 @@ sp_reduce_struct sp_PrepRem(long n)
 
 
 
-static inline
+inline
 long rem(unsigned long a, long n, sp_reduce_struct red) 
 {
-   unsigned long Q = MulHiUL(a, red.ninv);
+   unsigned long Q = ll_mul_hi(a, red.ninv);
    long r = a - Q*cast_unsigned(n);
    r = sp_CorrectExcess(r, n);
    return r;
 }
 
 
-static inline
+inline
 long rem(long a, long n, sp_reduce_struct red) 
 {
    unsigned long a0 = cast_unsigned(a) & ((1UL << (NTL_BITS_PER_LONG-1))-1);
@@ -981,20 +973,20 @@ long rem(long a, long n, sp_reduce_struct red)
 struct sp_reduce_struct { };
 
 
-static inline
+inline
 sp_reduce_struct sp_PrepRem(long n) 
 {
    return sp_reduce_struct();
 }
 
 
-static inline
+inline
 long rem(unsigned long a, long n, sp_reduce_struct red) 
 {
    return a % cast_unsigned(n);
 }
 
-static inline
+inline
 long rem(long a, long n, sp_reduce_struct red)
 {
    long r = a % n;
@@ -1019,14 +1011,14 @@ struct sp_ll_reduce_struct {
    unsigned long inv;
    long nbits;
 
-   sp_ll_reduce_struct() { }
+   sp_ll_reduce_struct() NTL_DEFAULT
 
    sp_ll_reduce_struct(unsigned long _inv, long _nbits) : inv(_inv), nbits(_nbits) { }
 
 };
 
 
-static inline sp_ll_reduce_struct
+inline sp_ll_reduce_struct
 make_sp_ll_reduce_struct(long n)
 {
    long nbits = NTL_BITS_PER_LONG - sp_CountLeadingZeros(n);
@@ -1038,13 +1030,13 @@ make_sp_ll_reduce_struct(long n)
 
 
 // computes remainder (hi, lo) mod d, assumes hi < d
-static inline long  
+inline long  
 sp_ll_red_21(unsigned long hi, unsigned long lo, long d, 
             sp_ll_reduce_struct dinv)
 {
    unsigned long H = 
       (hi << (NTL_BITS_PER_LONG-dinv.nbits)) | (lo >> dinv.nbits);
-   unsigned long Q = MulHiUL(H, dinv.inv) + H;
+   unsigned long Q = ll_mul_hi(H, dinv.inv) + H;
    unsigned long rr = lo - Q*cast_unsigned(d); // rr in [0..4*d)
    long r = sp_CorrectExcess(rr, 2*d); // r in [0..2*d)
    r = sp_CorrectExcess(r, d);
@@ -1052,7 +1044,7 @@ sp_ll_red_21(unsigned long hi, unsigned long lo, long d,
 }
 
 // computes remainder (x[n-1], ..., x[0]) mod d
-static inline long 
+inline long 
 sp_ll_red_n1(const unsigned long *x, long n, long d, sp_ll_reduce_struct dinv)
 {
    long carry = 0;
@@ -1063,7 +1055,7 @@ sp_ll_red_n1(const unsigned long *x, long n, long d, sp_ll_reduce_struct dinv)
 } 
 
 // computes remainder (x2, x1, x0) mod d, assumes x2 < d
-static inline long 
+inline long 
 sp_ll_red_31(unsigned long x2, unsigned long x1, unsigned long x0,
            long d, sp_ll_reduce_struct dinv)
 {
@@ -1075,13 +1067,13 @@ sp_ll_red_31(unsigned long x2, unsigned long x1, unsigned long x0,
 // normalized versions of the above: assume NumBits(d) == NTL_SP_NBITS
 
 // computes remainder (hi, lo) mod d, assumes hi < d
-static inline long  
+inline long  
 sp_ll_red_21_normalized(unsigned long hi, unsigned long lo, long d, 
             sp_ll_reduce_struct dinv)
 {
    unsigned long H = 
       (hi << (NTL_BITS_PER_LONG-NTL_SP_NBITS)) | (lo >> NTL_SP_NBITS);
-   unsigned long Q = MulHiUL(H, dinv.inv) + H;
+   unsigned long Q = ll_mul_hi(H, dinv.inv) + H;
    unsigned long rr = lo - Q*cast_unsigned(d); // rr in [0..4*d)
    long r = sp_CorrectExcess(rr, 2*d); // r in [0..2*d)
    r = sp_CorrectExcess(r, d);
@@ -1089,7 +1081,7 @@ sp_ll_red_21_normalized(unsigned long hi, unsigned long lo, long d,
 }
 
 // computes remainder (x[n-1], ..., x[0]) mod d
-static inline long 
+inline long 
 sp_ll_red_n1_normalized(const unsigned long *x, long n, long d, sp_ll_reduce_struct dinv)
 {
    long carry = 0;
@@ -1100,7 +1092,7 @@ sp_ll_red_n1_normalized(const unsigned long *x, long n, long d, sp_ll_reduce_str
 } 
 
 // computes remainder (x2, x1, x0) mod d, assumes x2 < d
-static inline long 
+inline long 
 sp_ll_red_31_normalized(unsigned long x2, unsigned long x1, unsigned long x0,
            long d, sp_ll_reduce_struct dinv)
 {
@@ -1117,7 +1109,7 @@ sp_ll_red_31_normalized(unsigned long x2, unsigned long x1, unsigned long x0,
 struct sp_ll_reduce_struct { };
 
 
-static inline sp_ll_reduce_struct
+inline sp_ll_reduce_struct
 make_sp_ll_reduce_struct(long n)
 {
    return sp_ll_reduce_struct();
